/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef VA_TYPES_H
#define VA_TYPES_H

#define TBUF (BUFSIZ/64)

#define TRUE  (1)
#define FALSE (0)

#define VA_ZERO_PLUS  (0x0000000000000000)
#define VA_ZERO_MINUS (0x8000000000000000)

#define LETTERSPACE_READ (0)
#define COLORSPACE_READ  (1)
#define MODE_HAPLOID    (1)
#define MODE_DIPLOID    (2)
#define SRC_SIM         (1)
#define SRC_REAL        (2)

#define MIN(_a, _b) ((_a) > (_b) ? (_b) : (_a))
#define MAX(_a, _b) ((_a) < (_b) ? (_b) : (_a))


typedef unsigned char VA_UINT8;
typedef char VA_INT8;
typedef unsigned short VA_UINT16;
typedef short VA_INT16;
typedef unsigned int VA_UINT32;
typedef int VA_INT32;
typedef double VA_DOUBLE;

typedef struct SnpData_t SnpData_t;
struct SnpData_t
{
    VA_INT32    chr;
    VA_INT32    pos;
    char      * hap1;
    char      * hap2;
    char      * snp;
    SnpData_t * next;
};

typedef struct VA_VecI32 VA_VecI32;
struct VA_VecI32
{
    size_t     size;
    VA_INT32 * data;
};

typedef struct VA_VecU32 VA_VecU32;
struct VA_VecU32
{
    size_t      size;
    VA_UINT32 * data;
};

typedef struct VA_VecU8 VA_VecU8;
struct VA_VecU8
{
    size_t     size;
    VA_UINT8 * data;
};

typedef struct VA_MatU8 VA_MatU8;
struct VA_MatU8
{
    size_t     size_x;
    size_t     size_y;
    VA_UINT8 * data;
};

typedef struct VA_MatI32 VA_MatI32;
struct VA_MatI32
{
    size_t     size_x;
    size_t     size_y;
    VA_INT32 * data;
};

typedef struct VA_MatU32 VA_MatU32;
struct VA_MatU32
{
    size_t      size_x;
    size_t      size_y;
    VA_UINT32 * data;
};

typedef struct VA_VecDouble VA_VecDouble;
struct VA_VecDouble
{
    size_t      size;
    VA_DOUBLE * data;
};

typedef struct VA_MatDouble VA_MatDouble;
struct VA_MatDouble
{
    size_t      size_x;
    size_t      size_y;
    VA_DOUBLE * data;
};

typedef struct VA_Vector VA_Vector;
struct VA_Vector
{
    size_t   size;
    void   * data;
};

typedef struct Covmaps_t Covmaps_t;
struct Covmaps_t
{
    VA_MatI32     coverage;
    VA_MatDouble  colors_positive;
    VA_MatDouble  colors_negative;
    VA_MatDouble  col_em;
    VA_MatDouble  noncol_em;
    VA_MatDouble  letters_positive;
    VA_MatDouble  letters_negative;
    VA_MatDouble  let_em;
    VA_MatDouble  nonlet_em;
};

typedef struct SnpGroup_t SnpGroup_t;
struct SnpGroup_t
{
    VA_VecI32    range;
    VA_VecI32    oldrange;
    VA_VecU8     hap1;
    VA_VecU8     hap2;
    VA_VecU8     is_snp;
    VA_VecU8     indel;
    VA_INT32     nr_snp;
    VA_Vector    snps;
    VA_INT32     insertions;
    VA_VecU8     chr;
    VA_UINT32    refindex;
    VA_UINT32    refstart;
    VA_UINT32    changed;
    Covmaps_t    covmap;
    VA_MatDouble maxvals_2;
    VA_MatDouble top;
    VA_MatU8     ch_list;
    VA_MatU8     lmat;
    VA_MatU8     cmat;
    VA_MatU32    sI;
    VA_VecI32    readstarts;
    VA_VecU8     ref;
};

typedef struct SnpGroups_t SnpGroups_t;
struct SnpGroups_t
{
    VA_INT32     size;
    SnpGroup_t * data;
};

typedef struct ReadsNode_t ReadsNode_t;
struct ReadsNode_t
{
    char      * name;
    char        strand;
    //VA_UINT32   chrstart;
    //VA_UINT32   chrend;
    VA_UINT32	cs_chrstart;
    VA_UINT32	cs_chrend;
    VA_UINT32	ls_chrstart;
    VA_UINT32	ls_chrend;
    VA_VecU8    ls_seq;
    VA_VecU8	ls_qvals;
    VA_VecU8	cs_qvals;
    VA_VecU8    cs_seq;
    VA_VecU8   qvals;
    VA_VecI32   ins;
    VA_VecI32	ls_ins;
    VA_VecI32	cs_ins;
    //VA_UINT32   seqtype;
    int		isLetterspace;
    struct 
    {
        VA_UINT32     haplome;
    } extras;
};

typedef struct CurrReads_t CurrReads_t;
struct CurrReads_t
{
    char   Rline[BUFSIZ];
    char   Tline[BUFSIZ];
    char   Gline[BUFSIZ];
    char   Qline[BUFSIZ];
    char   sline[BUFSIZ];
    char   Xline[BUFSIZ];
};

typedef struct Fasta_t Fasta_t;
struct Fasta_t 
{
    char     * hdr;
    VA_VecU8   seq;
    VA_VecU8   ref;
};

typedef struct CData_t CData_t;
struct CData_t 
{
    int       src;
    double    coverage;
    double    hap_bias;
    int       mode;
 
    /*TODO: fix the constants */
    double    error_mat_ls[1000];
    double    error_mat_cs[1000];
    VA_VecU8  ref;
    VA_Vector reads;
    VA_VecI32 readstarts;
};

typedef struct Comb_t Comb_t;
struct Comb_t
{
    VA_UINT8 c[5][8];
    VA_UINT8 l[5][8];
};


typedef struct Maps_t Maps_t;
struct Maps_t
{
    VA_MatU8 full_letter;
    VA_MatU8 call_letter;
    VA_MatU8 call_color;
  VA_VecU32 symm;
};

typedef struct VA_InternalStats VA_InternalStats;
struct VA_InternalStats {
    VA_VecU32    bad_calls;
    VA_VecU32    fp;
    VA_VecU32    fn;
    VA_VecU32    tp;
    VA_VecU32    tn;
    VA_VecU32    fo;
    VA_MatU32    covpos;
};

typedef struct VA_Stats VA_Stats;
struct VA_Stats {
    VA_UINT32    bad_calls;
    VA_UINT32    ffp;
    VA_UINT32    fp;
    VA_UINT32    fn;
    VA_UINT32    tp;
    VA_UINT32    tn;
    VA_UINT32    ifp;
    VA_UINT32    ifn;
    VA_UINT32    itp;
    VA_UINT32    itn;
    VA_UINT32    total_bases;
    VA_UINT32    total_add;
    VA_UINT32    thr_snp_index;
    VA_DOUBLE    thr_p;
    char       * ref_name;
};

typedef struct SparseDNode_t SparseDNode_t;
struct SparseDNode_t
{
    int x;
    int y;
    VA_DOUBLE val;
};

#endif

