/*
 * VARiD: A Variation Detection Framework for Colorspace and Letterspace platforms
 * Contact: <varid at cs dot toronto dot edu>
 * Copyright 2010 the authors (see README).
 *
 * This file is part of VARiD.
 *
 * VARiD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VARiD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VARiD.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef VARID_H
#define VARID_H
//define some flags, which are actually in run_wrap.c
extern int no_heuristics;
extern double gap_error;
extern double snp_rate;
extern double call_threshold;
extern double log_cutoff;
extern int incoming[1600][1600]; extern int incoming_used[1600];
extern int outgoing[1600][1600]; extern int outgoing_used[1600];
extern int get_bp_freq;
extern double freq[4];
extern int min_coverage;
extern int err_coverage;
extern double ls_err;
extern double cs_err;
extern FILE * save_learned_file;
extern FILE * load_learned_file;
extern double min_ls_err;
extern double min_cs_err;
extern double max_ls_err;
extern double max_cs_err;
extern char state_list[40][2];
extern double insert_open;
extern double insert_extend;
extern double deletion_open;
extern double deletion_extend;
extern FILE * output_file;
extern FILE * alignments_file;
extern char * alignments_filename;
extern FILE * reference_file;
extern char * reference_filename;
extern char * temp_directory;
extern char * donor_name;
extern int viterbi;
extern int poisson;
extern int normal;
extern int rank_sum;
extern double normal_std;
extern double log_coefficient_ls;
extern double log_coefficient_cs;
extern long long predict_start;
extern long long predict_end;
extern long long hmm_start;
extern long long hmm_end;
extern int haploid;
extern int drop_bad_reads;
extern int number_of_threads;
extern int binomial;
extern int format;
extern int report_all;
extern FILE * realignments_file;
extern FILE * read_alignments_file;
extern int no_header;
#define MAX_QVAL	255
#define GAP_QVAL	MAX_QVAL-1

#define FORMAT_PB	1
#define FORMAT_VCF	2

#include "va_types.h"
void print_matrix_double(FILE * f, VA_DOUBLE * data, int size_x, int size_y, VA_UINT8 * ref, VA_UINT8 * states) ;
void print_matrix_int_raw(FILE * f, VA_INT32 * data, int size_x, int size_y);
void print_matrix_double_raw(FILE * f, VA_DOUBLE * data, int size_x, int size_y);
void print_genome(FILE * f, VA_VecU8 ref);

extern char * error_strings[];


void varid_single();

void split_reads(
    char       ** filenames,
    SnpGroups_t * snpgroup,
    VA_INT32   ** fid_reads,
    VA_Vector     fastaVec);

void get_reads(
    char       * mapfile,
    VA_UINT32    nr_reads,
    VA_Vector  * outreads,
    VA_VecI32  * outinsvec,
    Fasta_t * f,
    VA_VecU8 * qval_ordering);

void process_read(
    char * sam_line,
    ReadsNode_t * read,
    VA_VecI32   * insvec,
    Fasta_t * f,
    VA_VecU8 * qval_ordering);

VA_Vector align_reads(
    VA_VecU8  ref,
    VA_Vector reads,
    VA_VecI32 ins_vec);

CData_t real_cdata_prep(
    int       isDip);

void varid(
    CData_t        cdata,
    VA_INT32       ref_pos,
    Maps_t       * maps,
    Covmaps_t    * covmap,
    VA_MatDouble * p,
    VA_VecU8 * qval_ordering);

void get_transitions(
    Comb_t         comb,
    VA_MatDouble * big_mat,
    Maps_t       * maps);

Covmaps_t varid_covmaps(
    CData_t cdata,
    VA_MatI32 * letter_qvals,
    VA_MatI32 * color_qvals,
    VA_VecU8 * qval_ordering,
    int unique_qvals);

VA_MatDouble varid_emissions(
    VA_VecU8	 ref,
    Covmaps_t    covmap,
    VA_MatI32 * letter_qvals,
    VA_MatI32 * color_qvals,
    VA_VecU8 * qval_ordering,
    int unique_qvals);

VA_DOUBLE varid_forward(
    VA_MatDouble   trans_mat,
    CData_t        cdata,
    VA_MatDouble   e,
    Maps_t         maps,
    VA_VecI32      window,
    VA_MatDouble * f,
    VA_VecDouble * s);

VA_DOUBLE varid_viterbi(
    VA_MatDouble   trans_mat,
    CData_t        cdata,
    VA_MatDouble   e,
    Maps_t         maps,
    VA_VecI32      window,
    VA_MatDouble * viterbi_prob);

VA_DOUBLE varid_backward(
    VA_MatDouble   trans_mat,
    CData_t        cdata,
    VA_MatDouble   e,
    Maps_t         maps,
    VA_VecI32      window,
    VA_MatDouble * b,
    VA_VecDouble * s);

void predict(
    VA_MatDouble p,
    Maps_t         maps,
    VA_MatDouble * maxvals_2,
    VA_MatU8     * lmat,
    VA_MatU8     * cmat);

void destroy_state(
    CData_t        cdata,
    VA_MatU8       cmat,
    VA_Vector      reads,
    VA_VecI32      readstarts,
    Covmaps_t      covmap,
    VA_VecDouble   thr_vec,
    VA_MatU8       lmat,
    VA_MatU8     * ch_list,
    VA_MatDouble * tot,
    VA_MatDouble * top2);

void get_pred(
    Covmaps_t      covmap,
    Maps_t         maps,
    VA_MatU8       ch_list,
    VA_MatDouble   maxvals_2,
    VA_MatU8       lmat,
    VA_VecU8       hap1,
    VA_VecU8       hap2,
    VA_VecU8     * pred1_whole,
    VA_VecU8     * pred2_whole);

VA_InternalStats compare_grp(
    VA_Vector    rha_real,
    VA_VecU8     hap1,
    VA_VecU8     hap2,
    VA_VecU8     ref,
    Covmaps_t    covmap);

void writeVCF(
    VA_UINT32    startpos,
    const char * ref_name,
    VA_VecU8     hap1,
    VA_VecU8     hap2,
    VA_VecU8     ref,
    VA_MatDouble maxvals_2,
    Covmaps_t    covmap,
    VA_MatU8 cmat,
    VA_MatU8 lmat);
void writePrettybase(
    VA_UINT32    startpos,
    const char * ref_name,
    VA_VecU8     hap1,
    VA_VecU8     hap2,
    VA_VecU8     ref,
    VA_MatDouble maxvals_2);

VA_VecU8 va_reallocVecU8(
    VA_VecU8 vec,
    size_t   n);

VA_VecU32 va_reallocVecU32(
    VA_VecU32 vec,
    size_t    n);

VA_VecI32 va_reallocVecI32(
    VA_VecI32 vec,
    size_t    n);

VA_VecU8 va_allocVecU8(
    size_t  size);

VA_VecI32 va_allocVecI32(
    size_t  size);

VA_VecU32 va_allocVecU32(
    size_t  size);

VA_VecDouble va_allocVecDouble(
    size_t  size);

VA_MatDouble va_allocMatDouble(
    size_t  size_x,
    size_t  size_y);

VA_MatI32 va_allocMatI32(
    size_t  size_x,
    size_t  size_y);

VA_MatU32 va_allocMatU32(
    size_t  size_x,
    size_t  size_y);

VA_MatU8 va_allocMatU8(
    size_t  size_x,
    size_t  size_y);

void fliplrU8(
    VA_VecU8 toFlip);

void fliplrDouble(
    VA_VecDouble toFlip);

int va_readfasta(
    VA_Vector  * vfasta);

VA_VecI32 va_sortVecI32(
    VA_VecI32 vec);

void va_loadMatDouble(
    char         * filename,
    VA_MatDouble * p);

void va_saveMatDouble(
    char       * filename,
    VA_MatDouble p);

VA_VecU8 nuc2col(
    VA_VecU8 innuc);

char * va_basename(
    const char * file);

VA_VecU8 col2nuc(
    VA_UINT8 startletter,
    VA_VecU8 colorref);

VA_VecU8 nuc2nuc(
    VA_VecU8 innuc);

VA_VecU8 get_complement(
    VA_VecU8 str1);

void va_load(
    const char * filename,
    VA_Vector  *  reads,
    VA_VecI32  * insvec);

void va_save(
    const char  * filename,
    VA_Vector     reads,
    VA_VecI32     insvec);

VA_UINT8 * nuc2nr(
    const char * str,
    size_t       sz);

VA_VecI32 va_unique(
    VA_VecI32 ref);

#endif

