#!/usr/bin/python
import re
import sys



re_vcf=re.compile('^(?P<chrom>\S+)\s+(?P<pos>\d+)\s+(?P<id>\S+)\s+(?P<ref>\S+)\s+(?P<alt>\S+)\s+(?P<qual>\S+)\s+(?P<filters_string>\S+)\s+(?P<info>\S+)\s+(?P<format>\S+)\s+(?P<donor_values>\S+)\s*$')
re_filter=re.compile('^(?P<filter_name>[^#]\S+)\s+(?P<required>\S+)\s+(?P<expression>[^~]+)\s*[~]\s+(?P<description>.*)$')

def usage():
	print '%s vcf_in filters_filename vcf_out' % sys.argv[0]
	return

def print_vcf(vcf):
	ret=[vcf["chrom"],vcf["pos"],vcf["id"],vcf["ref"],vcf["alt"],vcf["qual"],vcf["filters_string"],';'.join(vcf["info"]),":".join(vcf["format"]),":".join(vcf["donor_values"])]
	return "\t".join(ret)

def parse_vcf(line):
	m=re_vcf.match(line)
	if m:
		d=m.groupdict()
		d["format"]=d['format'].split(':')
		d["donor_values"]=d["donor_values"].split(':')
		full_defs=[]
		d["env"]={'QUAL':d["qual"]}
		for x in range(len(d["format"])):
			d["env"][d["format"][x]]=d["donor_values"][x]
		d["info"]=d["info"].split(';')
		for x in d["info"]:
			temp=x.split('=')
			name=temp[0]
			rest="=".join(temp[1:])
			d["env"][name]=rest
		for key in d["env"]:
			d["env"][key]=d["env"][key].split(',')
			l=d["env"][key]
			for z in range(len(l)):
				try:
					temp=int(l[z])
					l[z]=temp
				except:
					try: 
						temp=float(l[z])
						l[z]=temp
					except:
						pass
			if len(l)==1:
				d["env"][key]=l[0]
					
		d["defined"]={}
		for v in d["env"].keys():
			d["defined"][v]=1
		return d
	return None

def parse_filter(line):
	m=re_filter.match(line)
	if m:
		d=m.groupdict()
		d["required"]=d["required"].split(',')
		return d	
	return None

def has_defined(required,defined):
	for r in required:
		if not defined.has_key(r):
			return False
	return True

if __name__=='__main__':
	if len(sys.argv)!=4:
		usage()
		sys.exit(1)
	vcf_in_filename=sys.argv[1]
	filters_in_filename=sys.argv[2]
	vcf_out_filename=sys.argv[3]
	
	loaded_filters=[]
	filters=[]
	filters_in=open(filters_in_filename,'r')
	line=filters_in.readline()
	while line:
		if line[0]!='#':
			f=parse_filter(line)
			if f:
				filters.append(f)
				loaded_filters.append(f["filter_name"])
		line=filters_in.readline()
	filters_in.close()
	print >> sys.stderr, "Successfully loaded %d filters,\n%s" % (len(loaded_filters),",".join(loaded_filters)) 
	
	headers={}
	to_print=[]
	chrom_line=""

	vcf_in=open(vcf_in_filename,'r')
	vcf_out=open(vcf_out_filename,'w')
	line=vcf_in.readline()
	while line:
		if line[0]!='#':
			entry=parse_vcf(line)
			if entry:
				filters_failed=[]
				for filter in filters:
					if has_defined(filter["required"],entry["defined"]):
						if not eval(filter["expression"],entry["env"]):
							filters_failed.append(filter["filter_name"])	
						#if eval(filter["expression"]
				if len(filters_failed)>0:
					filters_string=filters_failed[0]
					for filter_failed in filters_failed[1:]:
						filters_string+=";"+filter_failed
					entry["filters_string"]=filters_string
				else:
					entry["filters_string"]="PASS"
				to_print.append(print_vcf(entry))
		else:
			mod_line=line.replace('#','')
			line_list=mod_line.split('=')
			name=line_list[0]
			if name=='FILTER':
				pass
			if name in ['fileformat','fileDate','source','reference','phasing','INFO','FORMAT']:
				if not headers.has_key(line):
					headers[line]=1
			if name[:5]=='CHROM':
				chrom_line=line
		line=vcf_in.readline()
	vcf_in.close()
	headers=headers.keys()

	#the new FILTERS
	for filter in filters:
		headers.append('##FILTER=<ID='+filter["filter_name"]+',Description="'+filter["description"]+'">\n')
	headers.sort(reverse=True)
	
	vcf_out.write("".join(headers)+chrom_line+"\n".join(to_print)+"\n")
	vcf_out.close()
